const path = require('path')
const clc = require('cli-color')
const webpack = require('webpack')
const uglify = require('uglifyjs-webpack-plugin')
const htmlPlugin = require('html-webpack-plugin')
const CopyWebpackPlugin = require('copy-webpack-plugin')
// const extractTextPlugin = require("extract-text-webpack-plugin");
const VueLoaderPlugin = require('vue-loader/lib/plugin')

module.exports = {
    /* 入口文件(以及对象写法) */
    // entry: __dirname + '/src/main.js',
    entry: {
        /* 应用程序mian入口文件 */
        entry: __dirname + '/src/main.js',
        /* 第三方库入口文件 */
        vue: 'vue'
    },
    /* 出口文件 */
    output: {
        path: path.resolve(__dirname + '/dist'),
        filename: '[name].js',
        // publicPath: 'http://localhost:9999/static'
        library: 'todayVuePlugIn',
        libraryTarget: 'umd',
        umdNamedDefine: true
    },
    /* 开发调试 */
    // devtool: 'eval-source-map',
    devServer: {
        /* 启动哪个资源文件 */
        contentBase: path.resolve(__dirname + '/dist'),
        host: 'localhost',
        port: 9999,
        /* 服务压缩 */
        compress:true
    },
    /* 自动监听选项 */
    watchOptions: {
        //检测修改的时间，以毫秒为单位
        poll:1000, 
        //防止重复保存而发生重复编译错误。这里设置的500是半秒内重复保存，不进行打包操作
        aggregateTimeout:500, 
        //不监听的目录
        ignored:/node_modules/, 
    },
    module: {
        rules: [
            {
                test: /\.css$/,
                use: ['style-loader', 'css-loader', 'postcss-loader'],
                // use: extractTextPlugin.extract({
                //     use: 'css-loader',
                //     fallback: 'style-loader'
                // })
            },
            {
                test: /\.less$/,
                use: ['style-loader', 'css-loader', 'less-loader', 'postcss-loader']
            },
            /* 处理图片路径，把图片压缩到js中 */
            {
                test: /\.(png|gif|jpg|jpeg)$/,
                use: [{
                    loader: 'url-loader',
                    options: {
                        limit: 50000
                    }
                }],
            },
            /* 处理vue文件 */
            {
                test: /\.vue$/,
                use: ['vue-loader']
            }
        ]
    },
    plugins: [
        new uglify(),
        new htmlPlugin({
            /* 压缩html */
            minify:{
                removeAttributeQuotes:false
            },
            /* 开发过程中避免缓存 */
            hash:true,
            /* 打包html的文件位置，将其打包到上面的出口地址 */
            template:'./src/index.html'
        }),
        /* 文件拷贝实现静态资源 */
        new CopyWebpackPlugin([{
            from: __dirname + '/src/static',
            to: __dirname + '/dist/static'
        }]),
        /* 分离css, /dist/index.css分离路径位置: 目前不支持webpack4+ */ 
        // new extractTextPlugin({
        //     filename: __dirname + '/dist/index.css',
        //     allChunks: true
        // })
        /* 为所打包的js打上注释 */
        new webpack.BannerPlugin(`
            小盛开vue脚手架,
            邮箱：354830981@qq.com
        `),
        /* vue解析插件 */
        new VueLoaderPlugin()
    ],
    optimization: {
        /* 第三方抽离 webpack4+ */
        splitChunks: {
            chunks: 'all'
        }
    }
}

if (process.env.type === 'build') {
    console.log(clc.green('我要打包咯~~~~')) 
    module.exports.devtool = ''
} else {
    module.exports.devtool = 'eval-source-map'
}